"""
file: constants.py
brief: Constants for the LDF class.
author: S. V. Paulauskas
date: February 05, 2019
"""
LDF_DATA_BUFFER_SIZE = 8193
LDF_DATA_BUFFER_SIZE_WITHOUT_HEADER = 8192
MINIMUM_CHUNK_SIZE = 20